@@ -8,7 +8,8 @@
 import atexit
 import logging
 from threading import Condition, RLock, Thread
-from time import sleep, time
+from time import time
+from tkinter import RAISED
 
 from guizero import App, Box, PushButton, Text
 
@@ -55,6 +56,8 @@ class LaunchGui(Thread):
         self.off_button = find_file("off_button.jpg")
         self.left_arrow = find_file("left_arrow.jpg")
         self.right_arrow = find_file("right_arrow.jpg")
+        self.engr_comm = find_file("walkie_talkie.png")
+        self.tower_comm = find_file("tower.png")
 
         self.counter = None
 
@@ -63,6 +66,7 @@ class LaunchGui(Thread):
         self.gantry_box = self.siren_box = self.klaxon_box = self.lights_box = None
         self.power_button = self.lights_button = self.siren_button = self.klaxon_button = None
         self.gantry_rev = self.gantry_fwd = None
+        self.comms_box = self.tower_comms = self.engr_comms = None
 
         self.track_on_req = CommandReq(TMCC1AuxCommandEnum.AUX1_OPT_ONE, track_id) if track_id else None
         self.power_on_req = CommandReq(TMCC1EngineCommandEnum.START_UP_IMMEDIATE, tmcc_id)
@@ -76,6 +80,8 @@ class LaunchGui(Thread):
         self.lights_off_req = CommandReq(TMCC1EngineCommandEnum.AUX2_OFF, tmcc_id)
         self.siren_req = CommandReq(TMCC1EngineCommandEnum.BLOW_HORN_ONE, tmcc_id)
         self.klaxon_req = CommandReq(TMCC1EngineCommandEnum.RING_BELL, tmcc_id)
+        self.engr_comm_req = CommandReq(TMCC1EngineCommandEnum.NUMERIC, tmcc_id, 2)
+        self.tower_comm_req = CommandReq(TMCC1EngineCommandEnum.NUMERIC, tmcc_id, 7)
         self.launch_15_req = CommandReq(TMCC1EngineCommandEnum.REAR_COUPLER, tmcc_id)
         self.launch_seq_act = CommandReq(TMCC1EngineCommandEnum.AUX1_OPTION_ONE, tmcc_id).as_action(duration=3.5)
 
@@ -123,79 +129,78 @@ class LaunchGui(Thread):
             self._dispatcher.subscribe(self, CommandScope.ENGINE, self.tmcc_id)
 
     def close(self) -> None:
-        with self._cv:
-            if not self._is_closed:
-                self._is_closed = True
-                self.app.after(10, self.app.destroy)
-                self.join()
+        if not self._is_closed:
+            self._is_closed = True
+            self.app.after(10, self.app.destroy)
+            self.join()
 
     def reset(self):
         self.close()
 
     def sync_gui_state(self) -> None:
         if self._monitored_state:
-            with self._cv:
-                # power on?
-                if self._monitored_state.is_started is True:
-                    self.do_power_on()
-                else:
-                    self.do_power_off()
-                # lights on?
-                self.sync_pad_lights()
+            # power on?
+            if self._monitored_state.is_started is True:
+                self.app.after(10, self.do_power_on)
+                self.app.after(20, self.sync_pad_lights)
+            else:
+                self.set_lights_on_icon()
+                self.app.after(10, self.do_power_off)
 
     def sync_pad_lights(self):
-        with self._cv:
-            if self._monitored_state.is_aux2 is True:
-                self.do_lights_on()
-            else:
-                self.do_lights_off()
+        if self._monitored_state.is_aux2 is True:
+            self.set_lights_off_icon()
+        else:
+            self.set_lights_on_icon()
 
     def __call__(self, cmd: CommandReq) -> None:
-        with self._cv:
-            # handle launch sequence differently
-            if cmd.command == TMCC1EngineCommandEnum.AUX1_OPTION_ONE:
-                if self._launch_seq_time_trigger is None:
-                    if self._last_cmd != cmd or (time() - self._last_cmd_at) > 5:
-                        self._launch_seq_time_trigger = time()
-                else:
-                    if self._last_cmd == cmd and (time() - self._launch_seq_time_trigger) > 3.1:
-                        if not self._is_countdown:
-                            self.do_launch(76, detected=True)
-                        self._launch_seq_time_trigger = None
-                self._last_cmd = cmd
-                self._last_cmd_at = time()
-                return
+        # handle launch sequence differently
+        if cmd.command == TMCC1EngineCommandEnum.AUX1_OPTION_ONE:
+            if self._launch_seq_time_trigger is None:
+                if self._last_cmd != cmd or (time() - self._last_cmd_at) > 5:
+                    self._launch_seq_time_trigger = time()
             else:
-                self._launch_seq_time_trigger = None
-            if cmd != self._last_cmd or (time() - self._last_cmd_at) >= 1.0:
-                self._last_cmd_at = time()
-                if cmd.command == TMCC1EngineCommandEnum.NUMERIC:
-                    if cmd.data in (3, 6):
-                        # mark launch pad as on and lights as on
-                        self.do_power_on()
-                        self.sync_pad_lights()
-                    elif cmd.data == 5:
-                        self.do_lights_off()
-                        self.do_power_off()
-                    elif cmd.data == 0:  # reset
-                        self.do_klaxon_off()
-                        if self._is_countdown:
-                            self.do_abort(detected=True)
-                        else:
-                            # reset causes engine to start up, check for that state change here
-                            self.sync_gui_state()
-                elif self.is_active():
-                    if cmd.command == TMCC1EngineCommandEnum.REAR_COUPLER:
-                        self.do_power_on()
-                        self.do_launch(15, detected=True, hold=False)
-                    elif cmd.command == TMCC1EngineCommandEnum.AUX2_OPTION_ONE:
-                        self.sync_pad_lights()
-                    elif cmd.command == TMCC1EngineCommandEnum.AUX2_ON:
-                        self.do_lights_on()
-                    elif cmd.command == TMCC1EngineCommandEnum.AUX2_OFF:
-                        self.do_lights_off()
-            # remember last command
+                if self._last_cmd == cmd and (time() - self._launch_seq_time_trigger) > 3.1:
+                    if not self._is_countdown:
+                        self.app.after(1, self.do_launch_detected, [80])
+                        self._launch_seq_time_trigger = None
             self._last_cmd = cmd
+            self._last_cmd_at = time()
+            return
+        else:
+            self._launch_seq_time_trigger = None
+        if cmd != self._last_cmd or (time() - self._last_cmd_at) >= 1.0:
+            self._last_cmd_at = time()
+            if cmd.command == TMCC1EngineCommandEnum.NUMERIC:
+                if cmd.data in (3, 6):
+                    # mark launch pad as on
+                    self.app.after(10, self.do_power_on)
+                    self.app.after(20, self.sync_pad_lights)
+                elif cmd.data == 5:  # power down
+                    self.app.after(10, self.set_lights_on_icon)
+                    self.app.after(20, self.do_power_off)
+                elif cmd.data == 0:  # reset
+                    if self._is_countdown:
+                        self.app.after(10, self.do_abort_detected)
+                    else:
+                        # reset causes engine to start up, check for that state change here
+                        self.app.after(10, self.sync_gui_state)
+                    self.app.after(20, self.do_klaxon_off)
+            elif self.is_active():
+                if cmd.command == TMCC1EngineCommandEnum.REAR_COUPLER:
+                    self.app.after(1, self.do_launch_detected, [15])
+                elif cmd.command == TMCC1EngineCommandEnum.AUX2_OPTION_ONE:
+                    self.app.after(1, self.sync_pad_lights)
+                elif cmd.command == TMCC1EngineCommandEnum.AUX2_ON:
+                    self.app.after(1, self.set_lights_off_icon)
+                elif cmd.command == TMCC1EngineCommandEnum.AUX2_OFF:
+                    self.app.after(1, self.set_lights_on_icon)
+                elif cmd.command == TMCC1EngineCommandEnum.BLOW_HORN_ONE:
+                    self.app.after(1, self.siren_sounded)
+                elif cmd.command == TMCC1EngineCommandEnum.RING_BELL:
+                    self.app.after(1, self.klaxon_sounded)
+        # remember last command
+        self._last_cmd = cmd
 
     def is_active(self) -> bool:
         return True if self._monitored_state and self._monitored_state.is_started is True else False
@@ -298,10 +303,8 @@ class LaunchGui(Thread):
             grid=[0, 1],
             height=self.s_72,
             width=self.s_72,
+            command=self.siren_req.send,
         )
-        self.siren_button.when_clicked = lambda x: self.siren_req.send()
-        self.siren_button.when_left_button_pressed = lambda _: self.toggle_sound(self.siren_button)
-        self.siren_button.when_left_button_released = lambda _: self.toggle_sound(self.siren_button)
 
         self.klaxon_box = klaxon_box = Box(lower_box, layout="grid", border=2, align="left")
         _ = Text(klaxon_box, text="Klaxon", grid=[0, 0], size=self.s_16, underline=True)
@@ -311,8 +314,33 @@ class LaunchGui(Thread):
             grid=[0, 1],
             height=self.s_72,
             width=self.s_72,
+            command=self.klaxon_req.send,
         )
-        self.klaxon_button.when_clicked = self.toggle_klaxon
+
+        if self.width > 480:
+            self.comms_box = comms_box = Box(lower_box, layout="grid", border=2, align="left")
+            _ = Text(comms_box, text="Comms", grid=[0, 0, 2, 1], size=self.s_16, underline=True)
+            self.engr_comms = PushButton(
+                comms_box,
+                image=self.engr_comm,
+                grid=[0, 1],
+                height=self.s_72,
+                width=self.s_72,
+                command=self.engr_comm_req.send,
+            )
+            self.engr_comms.tk.config(relief=RAISED)
+            self.engr_comms.bg = "white"
+
+            self.tower_comms = PushButton(
+                comms_box,
+                image=self.tower_comm,
+                grid=[1, 1],
+                height=self.s_72,
+                width=self.s_72,
+                command=self.tower_comm_req.send,
+            )
+            self.tower_comms.tk.config(relief=RAISED)
+            self.tower_comms.bg = "white"
 
         self.gantry_box = gantry_box = Box(lower_box, layout="grid", border=2, align="left")
         _ = Text(gantry_box, text="Gantry", grid=[0, 0, 2, 1], size=self.s_16, underline=True)
@@ -323,7 +351,7 @@ class LaunchGui(Thread):
             height=self.s_72,
             width=self.s_72,
         )
-        self.gantry_rev.when_clicked = lambda x: self.gantry_rev_req.send(repeat=2)
+        self.gantry_rev.when_clicked = lambda: self.gantry_rev_req.send(repeat=2)
 
         self.gantry_fwd = PushButton(
             gantry_box,
@@ -340,6 +368,8 @@ class LaunchGui(Thread):
         self.siren_box.disable()
         self.klaxon_box.disable()
         self.gantry_box.disable()
+        if self.comms_box:
+            self.comms_box.disable()
 
         # sync GUI with current state
         self.sync_gui_state()
@@ -347,6 +377,13 @@ class LaunchGui(Thread):
         # display GUI and start event loop; call blocks
         self.app.display()
 
+    def siren_sounded(self) -> None:
+        self.toggle_sound(self.siren_button)
+        self.siren_button.after(13000, self.toggle_sound, [self.siren_button])
+
+    def klaxon_sounded(self) -> None:
+        self.toggle_sound(self.klaxon_button)
+
     def update_counter(self, value: int = None):
         with self._cv:
             prefix = "-"
@@ -372,9 +409,13 @@ class LaunchGui(Thread):
             second = count % 60
             self.count.value = f"{prefix}{minute:02d}:{second:02d}"
 
+    def do_launch_detected(self, t_minus: int = 80):
+        self.do_launch(t_minus=t_minus, detected=True)
+
     def do_launch(self, t_minus: int = 80, detected: bool = False, hold=False):
         with self._cv:
             print(f"Launching: T Minus: {t_minus}")
+            self.do_power_on()
             if self._is_countdown:
                 self.count.cancel(self.update_counter)
             self._is_countdown = True
@@ -394,6 +435,9 @@ class LaunchGui(Thread):
             if not hold:
                 self.count.repeat(1090, self.update_counter)
 
+    def do_abort_detected(self):
+        self.do_abort(detected=True)
+
     def do_abort(self, detected: bool = False):
         """
         Abort launch sequence if counting down, initiate self-destruct if bird
@@ -439,19 +483,21 @@ class LaunchGui(Thread):
         self.label.value = "T-Minus"
         self.message.clear()
         self.cancel_flashing()
+        self.lower_box.hide()
         if self.power_button.image == self.on_button:
             self.do_power_on()
             if self.track_on_req:
                 self.track_on_req.send()
-                sleep(0.5)
-            self.power_on_req.send()
+            self.power_on_req.send(delay=0.5)
         else:
             self.do_power_off()
             self.power_off_req.send()
         self.power_button.height = self.power_button.width = self.s_72
+        self.lower_box.show()
 
     def do_power_off(self):
         with self._cv:
+            self.lights_off_req.send(repeat=2)
             self.cancel_flashing()
             if self._is_countdown:
                 self.count.cancel(self.update_counter)
@@ -464,6 +510,8 @@ class LaunchGui(Thread):
             self.siren_box.disable()
             self.klaxon_box.disable()
             self.gantry_box.disable()
+            if self.comms_box:
+                self.comms_box.disable()
 
     def do_power_on(self):
         with self._cv:
@@ -475,34 +523,27 @@ class LaunchGui(Thread):
             self.siren_box.enable()
             self.klaxon_box.enable()
             self.gantry_box.enable()
+            if self.comms_box:
+                self.comms_box.enable()
             self.sync_pad_lights()
             if self._is_countdown:
                 self.launch.disable()
 
-    def do_lights_on(self):
-        with self._cv:
-            if self.lights_button.image != self.off_button:
-                self.lights_button.image = self.off_button
-                self.lights_button.height = self.lights_button.width = self.s_72
+    def set_lights_off_icon(self):
+        if self.lights_button.image != self.off_button:
+            self.lights_button.image = self.off_button
+            self.lights_button.height = self.lights_button.width = self.s_72
 
-    def do_lights_off(self):
-        with self._cv:
-            if self.lights_button.image != self.on_button:
-                self.lights_button.image = self.on_button
-                self.lights_button.height = self.lights_button.width = self.s_72
+    def set_lights_on_icon(self):
+        if self.lights_button.image != self.on_button:
+            self.lights_button.image = self.on_button
+            self.lights_button.height = self.lights_button.width = self.s_72
 
     def toggle_lights(self):
-        with self._cv:
-            if self.lights_button.image == self.on_button:
-                self.do_lights_on()
-                self.lights_on_req.send(repeat=2)
-            else:
-                self.do_lights_off()
-                self.lights_off_req.send(repeat=2)
-
-    def toggle_klaxon(self) -> None:
-        self.klaxon_req.send()
-        self.toggle_sound(self.klaxon_button)
+        if self._monitored_state.is_aux2:
+            self.lights_off_req.send(repeat=2)
+        else:
+            self.lights_on_req.send(repeat=2)
 
     def do_klaxon_off(self):
         self.klaxon_button.image = self.siren_off
@@ -510,8 +551,10 @@ class LaunchGui(Thread):
 
     def toggle_sound(self, button: PushButton):
         if button.enabled is True:
+            self.lower_box.hide()
             if button.image == self.siren_off:
                 button.image = self.siren_on
             else:
                 button.image = self.siren_off
             button.height = button.width = self.s_72
+            self.lower_box.show()
