@@ -14,20 +14,20 @@ from enum import IntEnum, unique
 from math import floor
 from typing import Dict, List, Tuple
 
-from .constants import PdiCommand, PDI_SOP, PDI_EOP, D4Action
 from ..db.comp_data import (
-    CompDataMixin,
+    SCOPE_TO_COMP_MAP,
     CompData,
     CompDataHandler,
-    SCOPE_TO_COMP_MAP,
+    CompDataMixin,
 )
-from .base3_component import ConsistComponent
-from .pdi_req import PdiReq, SCOPE_TO_RECORD_LENGTH, LIONEL_ENGINE_RECORD_LENGTH
 from ..db.component_state import ComponentState
 from ..protocol.command_def import CommandDefEnum
 from ..protocol.command_req import CommandReq
-from ..protocol.constants import CommandScope, CONTROL_TYPE, SOUND_TYPE, LOCO_TYPE, LOCO_CLASS, Mixins
+from ..protocol.constants import CONTROL_TYPE, LOCO_CLASS, LOCO_TYPE, SOUND_TYPE, CommandScope, Mixins
 from ..protocol.tmcc2.tmcc2_constants import TMCC2EngineCommandEnum
+from .base3_component import ConsistComponent
+from .constants import PDI_EOP, PDI_SOP, D4Action, PdiCommand
+from .pdi_req import LIONEL_ENGINE_RECORD_LENGTH, SCOPE_TO_RECORD_LENGTH, PdiReq
 
 log = logging.getLogger(__name__)
 
@@ -117,6 +117,15 @@ SCOPE_TO_RECORD_TYPE_MAP = {s: p for p, s in RECORD_TYPE_MAP.items()}
 
 
 class BaseReq(PdiReq, CompDataMixin):
+    # noinspection PyTypeChecker,PyUnusedLocal
+    @classmethod
+    def request_config(cls, state: ComponentState, cmd: CommandReq) -> BaseReq:
+        if state.scope in {CommandScope.ENGINE, CommandScope.TRAIN}:
+            state_changes = BaseReq.update_eng(cmd)
+            for state_change in state_changes:
+                state_change.send()
+        return cls(state.address, pdi_command=PdiCommand.BASE_MEMORY, scope=state.scope)
+
     @classmethod
     def update_speed(cls, address: int, speed: int, scope: CommandScope = CommandScope.ENGINE) -> BaseReq:
         if scope == CommandScope.TRAIN:
@@ -132,6 +141,10 @@ class BaseReq(PdiReq, CompDataMixin):
         data: int | None = None,
         scope: CommandScope = CommandScope.ENGINE,
     ) -> List[BaseReq] | None:
+        #
+        # Given a TMCC command, determine how it indirectly impacts engine state and
+        # return a list of commands to send to the Base 3 to set that state
+        #
         from ..db.component_state_store import ComponentStateStore
 
         pkgs = []
@@ -140,12 +153,11 @@ class BaseReq(PdiReq, CompDataMixin):
             address = cmd.address
             data = cmd.data
             scope = cmd.scope
-            cur_state = ComponentStateStore.build().get_state(scope, address, False)
+            cur_state = ComponentStateStore.get_state(scope, address, False)
 
             # special case numeric commands
             if state.name == "NUMERIC":
                 if data in {3, 6}:  # RPM up/down
-                    state = TMCC2EngineCommandEnum.DIESEL_RPM
                     if cur_state and cur_state.rpm is not None:
                         cur_rpm = cur_state.rpm
                         cur_labor = cur_state.labor if cur_state.labor else 12
@@ -260,7 +272,7 @@ class BaseReq(PdiReq, CompDataMixin):
         data_bytes: bytes | None = None,
     ) -> None:
         super().__init__(data, pdi_command)
-        if self.pdi_command.is_base is False:
+        if not self.pdi_command.is_base:
             raise ValueError(f"Invalid PDI Base Request: {data}")
         self._status = self._firmware_high = self._firmware_low = None
         self._route_throw_rate = self._name = self._number = None
@@ -396,8 +408,7 @@ class BaseReq(PdiReq, CompDataMixin):
                     self._data_length = data_length
                     self._data_bytes = data_bytes
             elif state:
-                from .. import TrainState
-                from .. import EngineState
+                from .. import EngineState, TrainState
                 from ..db.base_state import BaseState
 
                 self._status = 0
